/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package db4ounit;

import java.lang.reflect.*;
import java.util.*;

/**
 * @sharpen.ignore
 */
public class UnitTestMain {
	public static void main(String[] args) throws Exception {
		new UnitTestMain().runTests(args);
	}

	public final void runTests(String[] args) throws ClassNotFoundException,
			InstantiationException, IllegalAccessException {
		TestSuite suite = build(args);
		TestRunner runner=new TestRunner(suite);
		runner.run();
	}
	
	protected TestSuiteBuilder builder(Class[] clazzes) {
		return new ReflectionTestSuiteBuilder(clazzes);
	}

	private TestSuite build(String[] args)
			throws ClassNotFoundException, InstantiationException,
			IllegalAccessException {
		Vector plainTestMethods=new Vector();
		Vector plainTestClasses=new Vector();
		for(int idx=0;idx<args.length;idx++) {
			String testIdentifier = args[idx];
			int methodSeparatorIndex = testIdentifier.indexOf('#');
			if(methodSeparatorIndex>0) {
				String className=testIdentifier.substring(0,methodSeparatorIndex);
				String methodName=testIdentifier.substring(methodSeparatorIndex+1);
				TestMethod testMethod = testMethod(className, methodName);
				plainTestMethods.addElement(testMethod);
				continue;
			}
			Class curClazz=Class.forName(testIdentifier);
			if(TestCase.class.isAssignableFrom(curClazz)) {
				plainTestClasses.addElement(curClazz);
			}
		}
		Class[] plainTestClassesArray = new Class[plainTestClasses.size()];
		vectorToArray(plainTestClasses, plainTestClassesArray);
		TestSuiteBuilder classBuilder=builder(plainTestClassesArray);
		Test[] plainTestMethodArray=new Test[plainTestMethods.size()];
		vectorToArray(plainTestMethods, plainTestMethodArray);
		TestSuite methodSuite=new TestSuite(plainTestMethodArray);
		return new TestSuite(new Test[]{classBuilder.build(),methodSuite});
	}
	
	private void vectorToArray(Vector vector, Object[] array){
		int i = 0;
		Enumeration enumer = vector.elements();
		while(enumer.hasMoreElements()){
			array[i++] = enumer.nextElement();
		}
	}
	private TestMethod testMethod(String className, String methodName)
			throws ClassNotFoundException, InstantiationException,
			IllegalAccessException {
		TestCase test = createTestInstance(className);
		Method method=null;
		Method[] methods = test.getClass().getMethods();
		for (int methodIdx = 0; methodIdx < methods.length; methodIdx++) {
			Method curMethod = methods[methodIdx];
			if(curMethod.getName().equals(methodName)) {
				method=curMethod;
			}
		}
		if(method==null) {
			return null;
		}
		return new TestMethod(test,method);
	}

	protected TestCase createTestInstance(String className) throws ClassNotFoundException,
			InstantiationException, IllegalAccessException {
		Class clazz=Class.forName(className);
		TestCase test=(TestCase)clazz.newInstance();
		return test;
	}
}
