/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package db4ounit.tests;

import java.util.Enumeration;

import db4ounit.Assert;
import db4ounit.Test;
import db4ounit.TestFailure;
import db4ounit.TestCase;
import db4ounit.TestResult;
import db4ounit.TestSuite;

public class FrameworkTestCase implements TestCase {
	public final static RuntimeException EXCEPTION = new RuntimeException();
	
	public void testRunsGreen() {
		TestResult result = new TestResult();
		new RunsGreen().run(result);
		Assert.isTrue(result.failures().size() == 0, "not green");
	}
	
	public void testRunsRed() {
		TestResult result = new TestResult();
		new RunsRed(EXCEPTION).run(result);
		Assert.isTrue(result.failures().size() == 1, "not red");
	}
	
	public void testTestSuite() {
		runTestAndExpect(new TestSuite(new Test[]{new RunsGreen()}),0);
		runTestAndExpect(new TestSuite(new Test[]{new RunsRed(EXCEPTION)}),1);
		runTestAndExpect(new TestSuite(new Test[]{new RunsGreen(),new RunsRed(EXCEPTION)}),1);
		runTestAndExpect(new TestSuite(new Test[]{new RunsRed(EXCEPTION),new RunsRed(EXCEPTION)}),2);
		runTestAndExpect(new TestSuite(new Test[]{new RunsRed(EXCEPTION),new RunsGreen()}),1);
		runTestAndExpect(new TestSuite(new Test[]{new RunsGreen(),new RunsGreen()}),0);
	}
	
	public static void runTestAndExpect(Test test,int expFailures) {
		runTestAndExpect(test,expFailures,true);
	}
	
	public static void runTestAndExpect(Test test,int expFailures, boolean checkException) {
		TestResult result = new TestResult();
		test.run(result);
		if (expFailures != result.failures().size()) {
			Assert.fail(result.failures().toString());
		}
		if (checkException) {
			for(Enumeration iter=result.failures().iterator(); iter.hasMoreElements();) {
				TestFailure failure = (TestFailure) iter.nextElement();
				Assert.isTrue(EXCEPTION.equals(failure.getFailure()));
			}
		}
	}
}
